include <imhdr.h>

define	OPPERATIONS	"|multiply|divide|"
define	OPMULTIPLY	1
define	OPDIVIDE	2
define	TOL1		 1.0e-7
define	TOL2		-1.0e-7

procedure t_quadscale ()

char	input[SZ_FNAME]		#TI Input image name.
char	output[SZ_FNAME]	#TI Output image name.

real	gain[4]			#TI Gain factor for each quadrant
int	op			#TI Multiply or divide by gain factors

int	i
pointer	in, out
char	buffer[SZ_LINE]

real	clgetr()
int	clgwrd()
pointer	immap()

begin

	# Open input image
	call clgstr ("input",  input,  SZ_FNAME)
	in = immap  (input, READ_ONLY, 0)

	# Open output image
	call clgstr ("output",  output,  SZ_FNAME)
	out = immap (output, NEW_COPY, in)
	IM_PIXTYPE(out) = TY_REAL

	# Get gain factors
	gain[1] = clgetr ("gain11")
	gain[2] = clgetr ("gain12")
	gain[3] = clgetr ("gain21")
	gain[4] = clgetr ("gain22")

	# Get direction of opperation
	op = clgwrd ("opperation", buffer, SZ_LINE, OPPERATIONS)

	switch (op) {
	case OPMULTIPLY:
	    call quadmult (in, out, gain)

	case OPDIVIDE:
	    # Check for zero gain --> divide by zero
	    do i = 1, 4 {
		if ((gain[i] < TOL1) && (gain[i] > TOL2)) {
		    call error (0, "Attempt to divide by gain value of zero")
		}
	    }
	    call quaddiv (in, out, gain)

	}

	call imunmap (in)
	call imunmap (out)
end

procedure quadmult (in, out, gain)

pointer	in				#I imio pointer for input image.
pointer	out				#I imio pointer for output image.
real	gain[4]				#I Array of gain factors.


pointer	inbuf, obuf
int	junk, nx, ny, iy
long	invec[IM_MAXDIM], ovec[IM_MAXDIM]

int	imgnlr(), impnlr()

begin
	nx = IM_LEN (in, 1) / 2
	ny = IM_LEN (in, 2) / 2

	# Setup start vector for sequential reads
	call amovkl (long(1), invec, IM_MAXDIM)
	call amovkl (long(1), ovec,  IM_MAXDIM)

	# Process first two quadrants
	do iy = 1, ny {
	    junk = imgnlr (in,  inbuf, invec)
	    junk = impnlr (out, obuf,  ovec) 
	    call amulkr (Memr[inbuf],    gain[1], Memr[obuf],    nx)
	    call amulkr (Memr[inbuf+nx], gain[2], Memr[obuf+nx], nx)
	}

	# Process second two quadrants
	do iy = 1, ny {
	    junk = imgnlr (in,  inbuf, invec)
	    junk = impnlr (out, obuf,  ovec) 
	    call amulkr (Memr[inbuf],    gain[3], Memr[obuf],    nx)
	    call amulkr (Memr[inbuf+nx], gain[4], Memr[obuf+nx], nx)
	}

end

procedure quaddiv (in, out, gain)

pointer	in				#I imio pointer for input image.
pointer	out				#I imio pointer for output image.
real	gain[4]				#I Array of gain factors.


pointer	inbuf, obuf
int	junk, nx, ny, iy
long	invec[IM_MAXDIM], ovec[IM_MAXDIM]

int	imgnlr(), impnlr()

begin
	nx = IM_LEN (in, 1) / 2
	ny = IM_LEN (in, 2) / 2

	# Setup start vector for sequential reads
	call amovkl (long(1), invec, IM_MAXDIM)
	call amovkl (long(1), ovec,  IM_MAXDIM)

	# Process first two quadrants
	do iy = 1, ny {
	    junk = imgnlr (in,  inbuf, invec)
	    junk = impnlr (out, obuf,  ovec) 
	    call adivkr (Memr[inbuf],    gain[1], Memr[obuf],    nx)
	    call adivkr (Memr[inbuf+nx], gain[2], Memr[obuf+nx], nx)
	}

	# Process second two quadrants
	do iy = 1, ny {
	    junk = imgnlr (in,  inbuf, invec)
	    junk = impnlr (out, obuf,  ovec) 
	    call adivkr (Memr[inbuf],    gain[3], Memr[obuf],    nx)
	    call adivkr (Memr[inbuf+nx], gain[4], Memr[obuf+nx], nx)
	}

end
