include <imhdr.h>
include "quad.h"

procedure t_quadsplit ()

char	input[SZ_FNAME]		#TI Input image name.
char	output[SZ_FNAME]	#TI Output image root name.
char	instrument[SZ_FNAME]	#TI Output image root name.

pointer	in, out1, out2, out3, out4, inbuf, obuf1, obuf2
int	junk, nx
long	invec[IM_MAXDIM], ovec1[IM_MAXDIM], ovec2[IM_MAXDIM]

pointer	immap(), quadmap()
bool	streq()
int	imgnls(), imgnll(), imgnlr, imgnld, imgnlx, hdmaccf()
int	impnls(), impnll(), impnlr, impnld, impnlx

begin

	# Open instrument file
	call clgstr    ("ccdred.instrument",  instrument,  SZ_FNAME)
	call hdmopen   (instrument)

	# Open input image
	call clgstr ("input",  input,  SZ_FNAME)
	in = immap  (input, READ_ONLY, 0)
	nx = IM_LEN (in, 1) / 2

	# Check to see if the image has already been trimmed and if so exit
	if (hdmaccf (in, "trim") == YES)
	    return 

	# Set-up section translation
	call quadinit  (in)

	# Open output images one for each quadrant
	call clgstr  ("output", output, SZ_FNAME)
	if (streq (output, ""))
	    call strcpy (input, output, SZ_FNAME)

	out1 = quadmap (output, 1, NEW_COPY, in)
	out2 = quadmap (output, 2, NEW_COPY, in)
	out3 = quadmap (output, 3, NEW_COPY, in)
	out4 = quadmap (output, 4, NEW_COPY, in)

	# Set trimsec biasec etc. for each quadrant
	call quadsection (out1, 1)
	call quadsection (out2, 2)
	call quadsection (out3, 3)
	call quadsection (out4, 4)

	# Setup start vector for sequential reads
	call amovkl (long(1), invec, IM_MAXDIM)

        switch (IM_PIXTYPE(in)) {
        case TY_SHORT:

	    # Copy first two quadrants
	    call amovkl (long(1), ovec1, IM_MAXDIM)
	    call amovkl (long(1), ovec2, IM_MAXDIM)
            while (impnls (out1, obuf1, ovec1) != EOF) {
                junk = impnls (out2, obuf2, ovec2)
                junk = imgnls (in,   inbuf, invec)
                call amovs (Mems[inbuf],     Mems[obuf1], nx)
                call amovs (Mems[inbuf+nx], Mems[obuf2], nx)
            }

	    # Copy second two quadrants
	    call amovkl (long(1), ovec1, IM_MAXDIM)
	    call amovkl (long(1), ovec2, IM_MAXDIM)
            while (impnls (out3, obuf1, ovec1) != EOF) {
                junk = impnls (out4, obuf2, ovec2)
                junk = imgnls (in,   inbuf, invec)
                call amovs (Mems[inbuf],     Mems[obuf1], nx)
                call amovs (Mems[inbuf+nx], Mems[obuf2], nx)
            }

        case TY_USHORT, TY_INT, TY_LONG:

	    # Copy first two quadrants
	    call amovkl (long(1), ovec1, IM_MAXDIM)
	    call amovkl (long(1), ovec2, IM_MAXDIM)
            while (impnll (out1, obuf1, ovec1) != EOF) {
                junk = impnll (out2, obuf2, ovec2)
                junk = imgnll (in,   inbuf, invec)
                call amovl (Meml[inbuf],     Meml[obuf1], nx)
                call amovl (Meml[inbuf+nx], Meml[obuf2], nx)
            }

	    # Copy second two quadrants
	    call amovkl (long(1), ovec1, IM_MAXDIM)
	    call amovkl (long(1), ovec2, IM_MAXDIM)
            while (impnll (out3, obuf1, ovec1) != EOF) {
                junk = impnll (out4, obuf2, ovec2)
                junk = imgnll (in,   inbuf, invec)
                call amovl (Meml[inbuf],     Meml[obuf1], nx)
                call amovl (Meml[inbuf+nx], Meml[obuf2], nx)
            }

        case TY_REAL:

	    # Copy first two quadrants
	    call amovkr (long(1), ovec1, IM_MAXDIM)
	    call amovkr (long(1), ovec2, IM_MAXDIM)
            while (impnlr (out1, obuf1, ovec1) != EOF) {
                junk = impnlr (out2, obuf2, ovec2)
                junk = imgnlr (in,   inbuf, invec)
                call amovr (Memr[inbuf],     Memr[obuf1], nx)
                call amovr (Memr[inbuf+nx], Memr[obuf2], nx)
            }

	    # Copy second two quadrants
	    call amovkr (long(1), ovec1, IM_MAXDIM)
	    call amovkr (long(1), ovec2, IM_MAXDIM)
            while (impnlr (out3, obuf1, ovec1) != EOF) {
                junk = impnlr (out4, obuf2, ovec2)
                junk = imgnlr (in,   inbuf, invec)
                call amovr (Memr[inbuf],     Memr[obuf1], nx)
                call amovr (Memr[inbuf+nx], Memr[obuf2], nx)
            }

        case TY_DOUBLE:

	    # Copy first two quadrants
	    call amovkd (long(1), ovec1, IM_MAXDIM)
	    call amovkd (long(1), ovec2, IM_MAXDIM)
            while (impnld (out1, obuf1, ovec1) != EOF) {
                junk = impnld (out2, obuf2, ovec2)
                junk = imgnld (in,   inbuf, invec)
                call amovd (Memd[inbuf],     Memd[obuf1], nx)
                call amovd (Memd[inbuf+nx], Memd[obuf2], nx)
            }

	    # Copy second two quadrants
	    call amovkd (long(1), ovec1, IM_MAXDIM)
	    call amovkd (long(1), ovec2, IM_MAXDIM)
            while (impnld (out3, obuf1, ovec1) != EOF) {
                junk = impnld (out4, obuf2, ovec2)
                junk = imgnld (in,   inbuf, invec)
                call amovd (Memd[inbuf],     Memd[obuf1], nx)
                call amovl (Memd[inbuf+nx], Memd[obuf2], nx)
            }

        case TY_COMPLEX:

	    # Copy first two quadrants
	    call amovkx (long(1), ovec1, IM_MAXDIM)
	    call amovkx (long(1), ovec2, IM_MAXDIM)
            while (impnll (out1, obuf1, ovec1) != EOF) {
                junk = impnlx (out2, obuf2, ovec2)
                junk = imgnlx (in,   inbuf, invec)
                call amovx (Memx[inbuf],     Memx[obuf1], nx)
                call amovx (Memx[inbuf+nx], Memx[obuf2], nx)
            }

	    # Copy second two quadrants
	    call amovkx (long(1), ovec1, IM_MAXDIM)
	    call amovkx (long(1), ovec2, IM_MAXDIM)
            while (impnlx (out3, obuf1, ovec1) != EOF) {
                junk = impnlx (out4, obuf2, ovec2)
                junk = imgnlx (in,   inbuf, invec)
                call amovx (Memx[inbuf],     Memx[obuf1], nx)
                call amovx (Memx[inbuf+nx], Memx[obuf2], nx)
            }

        default:
            call error (1, "unknown pixel datatype")
        }

	call imunmap (in)
	call imunmap (out1)
	call imunmap (out2)
	call imunmap (out3)
	call imunmap (out4)
	call hdmclose ()
end
